@@ -31,6 +31,13 @@ def create_parser() -> argparse.ArgumentParser:
         nargs="?",
         help="file containing the exported transactions from Charles Schwab",
     )
+    parser.add_argument(
+        "--schwab-award",
+        type=str,
+        default=None,
+        nargs="?",
+        help="file containing schwab award data for stock prices",
+    )
     parser.add_argument(
         "--trading212",
         type=str,
@@ -43,6 +50,13 @@ def create_parser() -> argparse.ArgumentParser:
         nargs="?",
         help="folder containing the exported transaction files from Morgan Stanley",
     )
+    parser.add_argument(
+        "--sharesight",
+        type=str,
+        nargs="?",
+        help="folder containing reports from Sharesight in CSV format",
+    )
+
     parser.add_argument(
         "--gbp-history",
         type=str,
@@ -50,13 +64,6 @@ def create_parser() -> argparse.ArgumentParser:
         nargs="?",
         help="monthly GBP/USD prices from HMRC",
     )
-    parser.add_argument(
-        "--schwab-award",
-        type=str,
-        default=None,
-        nargs="?",
-        help="file containing schwab award data for stock prices",
-    )
     parser.add_argument(
         "--initial-prices",
         type=str,
@@ -76,6 +83,12 @@ def create_parser() -> argparse.ArgumentParser:
         action="store_true",
         help="do not generate PDF report",
     )
+    parser.add_argument(
+        "--no-balance-check",
+        dest="balance_check",
+        action="store_false",
+        default=True,
+    )
     parser.add_argument(
         "--verbose",
         action="store_true",
