@@ -3,6 +3,7 @@
 from __future__ import annotations
 
 from collections import defaultdict
+from dataclasses import astuple
 import datetime
 import decimal
 from decimal import Decimal
@@ -54,11 +55,21 @@ def get_amount_or_fail(transaction: BrokerTransaction) -> Decimal:
     return amount
 
 
+# It is not clear how Schwab or other brokers round the dollar value,
+# so assume the values are equal if they are within $0.01.
+def _approx_equal(val_a: Decimal, val_b: Decimal) -> bool:
+    return abs(val_a - val_b) < Decimal("0.01")
+
+
 class CapitalGainsCalculator:
     """Main calculator class."""
 
     def __init__(
-        self, tax_year: int, converter: CurrencyConverter, initial_prices: InitialPrices
+        self,
+        tax_year: int,
+        converter: CurrencyConverter,
+        initial_prices: InitialPrices,
+        balance_check: bool = True,
     ):
         """Create calculator object."""
         self.tax_year = tax_year
@@ -68,6 +79,7 @@ class CapitalGainsCalculator:
 
         self.converter = converter
         self.initial_prices = initial_prices
+        self.balance_check = balance_check
 
     def date_in_tax_year(self, date: datetime.date) -> bool:
         """Check if date is within current tax year."""
@@ -104,8 +116,8 @@ class CapitalGainsCalculator:
                 raise PriceMissingError(transaction)
 
             amount = get_amount_or_fail(transaction)
-            calculated_amount = round_decimal(quantity * price + transaction.fees, 2)
-            if amount != -calculated_amount:
+            calculated_amount = quantity * price + transaction.fees
+            if not _approx_equal(amount, -calculated_amount):
                 raise CalculatedAmountDiscrepancyError(transaction, -calculated_amount)
             amount = -amount
 
@@ -151,8 +163,8 @@ class CapitalGainsCalculator:
 
         if price is None:
             raise PriceMissingError(transaction)
-        calculated_amount = round_decimal(quantity * price - transaction.fees, 2)
-        if amount != calculated_amount:
+        calculated_amount = quantity * price - transaction.fees
+        if not _approx_equal(amount, calculated_amount):
             raise CalculatedAmountDiscrepancyError(transaction, calculated_amount)
         add_to_list(
             disposal_list,
@@ -207,7 +219,11 @@ class CapitalGainsCalculator:
                     gbp_fees,
                     gbp_fees,
                 )
-            elif transaction.action in [ActionType.STOCK_ACTIVITY, ActionType.SPIN_OFF]:
+            elif transaction.action in [
+                ActionType.STOCK_ACTIVITY,
+                ActionType.SPIN_OFF,
+                ActionType.REINVEST_SHARES,
+            ]:
                 self.add_acquisition(portfolio, acquisition_list, transaction)
             elif transaction.action in [ActionType.DIVIDEND, ActionType.CAPITAL_GAIN]:
                 amount = get_amount_or_fail(transaction)
@@ -224,11 +240,16 @@ class CapitalGainsCalculator:
                 new_balance += amount
                 if self.date_in_tax_year(transaction.date):
                     interest += self.converter.to_gbp_for(amount, transaction)
+            elif transaction.action is ActionType.WIRE_FUNDS_RECEIVED:
+                amount = get_amount_or_fail(transaction)
+                new_balance += amount
+            elif transaction.action is ActionType.REINVEST_DIVIDENDS:
+                print(f"WARNING: Ignoring unsupported action: {transaction.action}")
             else:
                 raise InvalidTransactionError(
                     transaction, f"Action not processed({transaction.action})"
                 )
-            if new_balance < 0:
+            if self.balance_check and new_balance < 0:
                 msg = f"Reached a negative balance({new_balance})"
                 msg += f" for broker {transaction.broker} ({transaction.currency})"
                 msg += " after processing the following transactions:\n"
@@ -258,9 +279,9 @@ class CapitalGainsCalculator:
         date_index: datetime.date,
     ) -> list[CalculationEntry]:
         """Process single acquisition."""
-        acquisition_quantity, acquisition_amount, acquisition_fees = acquisition_list[
-            date_index
-        ][symbol]
+        acquisition_quantity, acquisition_amount, acquisition_fees = astuple(
+            acquisition_list[date_index][symbol]
+        )
         original_acquisition_amount = acquisition_amount
         if symbol not in portfolio:
             portfolio[symbol] = (Decimal(0), Decimal(0))
@@ -280,7 +301,7 @@ class CapitalGainsCalculator:
                     bed_and_breakfast_quantity,
                     bed_and_breakfast_amount,
                     _bed_and_breakfast_fees,
-                ) = bed_and_breakfast_list[date_index][symbol]
+                ) = astuple(bed_and_breakfast_list[date_index][symbol])
                 assert bed_and_breakfast_quantity <= acquisition_quantity
                 acquisition_amount -= bed_and_breakfast_quantity * acquisition_price
                 acquisition_amount += bed_and_breakfast_amount
@@ -330,9 +351,9 @@ class CapitalGainsCalculator:
         date_index: datetime.date,
     ) -> tuple[Decimal, list[CalculationEntry]]:
         """Process single disposal."""
-        disposal_quantity, proceeds_amount, disposal_fees = disposal_list[date_index][
-            symbol
-        ]
+        disposal_quantity, proceeds_amount, disposal_fees = astuple(
+            disposal_list[date_index][symbol]
+        )
         original_disposal_quantity = disposal_quantity
         disposal_price = proceeds_amount / disposal_quantity
         current_quantity, current_amount = portfolio[symbol]
@@ -341,9 +362,9 @@ class CapitalGainsCalculator:
         calculation_entries = []
         # Same day rule is first
         if has_key(acquisition_list, date_index, symbol):
-            same_day_quantity, same_day_amount, _same_day_fees = acquisition_list[
-                date_index
-            ][symbol]
+            same_day_quantity, same_day_amount, _same_day_fees = astuple(
+                acquisition_list[date_index][symbol]
+            )
             available_quantity = min(disposal_quantity, same_day_quantity)
             if available_quantity > 0:
                 acquisition_price = same_day_amount / same_day_quantity
@@ -389,7 +410,7 @@ class CapitalGainsCalculator:
                         acquisition_quantity,
                         acquisition_amount,
                         _acquisition_fees,
-                    ) = acquisition_list[search_index][symbol]
+                    ) = astuple(acquisition_list[search_index][symbol])
 
                     bed_and_breakfast_quantity = Decimal(0)
                     if has_key(bed_and_breakfast_list, search_index, symbol):
@@ -397,7 +418,7 @@ class CapitalGainsCalculator:
                             bed_and_breakfast_quantity,
                             _bb_amount,
                             _bb_fees,
-                        ) = bed_and_breakfast_list[search_index][symbol]
+                        ) = astuple(bed_and_breakfast_list[search_index][symbol])
                     assert bed_and_breakfast_quantity <= acquisition_quantity
 
                     same_day_quantity = Decimal(0)
@@ -406,7 +427,7 @@ class CapitalGainsCalculator:
                             same_day_quantity,
                             _same_day_amount,
                             _same_day_fees,
-                        ) = disposal_list[search_index][symbol]
+                        ) = astuple(disposal_list[search_index][symbol])
                     assert same_day_quantity <= acquisition_quantity
 
                     # This can be some management fee entry or already used
@@ -534,7 +555,7 @@ class CapitalGainsCalculator:
         calculation_log: CalculationLog = {}
         for date_index in (
             begin_index + datetime.timedelta(days=x)
-            for x in range(0, (end_index - begin_index).days)
+            for x in range(0, (end_index - begin_index).days + 1)
         ):
             if date_index in acquisition_list:
                 for symbol in acquisition_list[date_index]:
@@ -568,12 +589,14 @@ class CapitalGainsCalculator:
                         disposal_count += 1
                         transaction_disposal_proceeds = disposal_list[date_index][
                             symbol
-                        ][1]
+                        ].amount
                         disposal_proceeds += transaction_disposal_proceeds
                         allowable_costs += (
                             transaction_disposal_proceeds - transaction_capital_gain
                         )
-                        transaction_quantity = disposal_list[date_index][symbol][0]
+                        transaction_quantity = disposal_list[date_index][
+                            symbol
+                        ].quantity
                         LOGGER.debug(
                             "DISPOSAL on %s of %s, quantity %d, capital gain $%s",
                             date_index,
@@ -640,12 +663,14 @@ def main() -> int:
 
     # Read data from input files
     broker_transactions = read_broker_transactions(
-        args.schwab, args.schwab_award, args.trading212, args.mssb
+        args.schwab, args.schwab_award, args.trading212, args.mssb, args.sharesight
     )
     converter = CurrencyConverter(read_gbp_prices_history(args.gbp_history))
     initial_prices = InitialPrices(read_initial_prices(args.initial_prices))
 
-    calculator = CapitalGainsCalculator(args.year, converter, initial_prices)
+    calculator = CapitalGainsCalculator(
+        args.year, converter, initial_prices, balance_check=args.balance_check
+    )
     # First pass converts broker transactions to HMRC transactions.
     # This means applying same day rule and collapsing all transactions with
     # same type within the same day.
